/*
 * FILENAME:    controller.js
 * VERSION:     0.256 (10/18/2011)
 * AUTHOR:      Tiffany Chao (tchao@cs.umd.edu)
 */

var controller = function(controller, jQuery) {
////// visible ////////////////////////////////////////////////////////////////
    var visible = {};

    
    visible.init = function() {
        // fetch data to display
        viewData = model.viewData(true, true);
        
        // prepare interface
        populateDOM();
        prepareTransitions();
        prepareHandlers();
        
        // browser-specific css oddity
        if ($.browser.mozilla) {
            $("#content td").css("height", "3.5em");
        }
        
        // indicate current dataset
        var dataset = utils.getStorageItem("dataset");
        
        if (dataset) {
            $("#dataset").val(dataset);
        } else {
            $("#dataset").val(model.DATASET_DEFAULT);
        }
        redraw();
    };
    
    
    // pseudo constants ///////////////////////////////////////////////////////
    var LIST_1_UNIQUE_INDEX = 0;
    var LIST_1_INDEX = 1;
    var LIST_IDENTICAL_INDEX = 2;
    var LIST_2_INDEX = 3;
    var LIST_2_UNIQUE_INDEX = 4;
    var NUM_ROWS = {};
    var NUM_COLUMNS = 5;
    
    var STATE_SEPARATE = 0;
    var STATE_IDENTICAL = 1;
    var STATE_UNIQUE = 2;
    var STATE_SIMILAR = 3;
    var STATE_COMPACT = 4;
    var NUM_STATES = 5;
    
    var WHITE = "#ffffff";
    var GRAY = "#eeeeee";
    
    
    // data ///////////////////////////////////////////////////////////////////
    var viewData = [];
    
    var headers = [];
    var previousHeader = "";
    var currentHeader = "";
    
    var state = STATE_SEPARATE;
    var positions = {};
    var keyframes = {};
    var animationSpeed = 800;
    var buffer = animationSpeed * 0.75;
    
    
    // initialization /////////////////////////////////////////////////////////
    function reset() {
        // complete reset (ignore cache)
        location.reload(true);
    }
    
    function prepareTransitions() {
        // calculate column positions exactly once; they won't change
        calculateColumns();
        
        // row positions, however, will
        calculatePositions();
    }
    
    function populateDOM() {
        populateLists($("#content"));
    }
    
    function populateLists($container) {
        populateItems($container);
        populateBackdrop($container);
    }
    
    function populateItems($container) {
        // prepare item container
        var $items = $("<div></div>");
        $items.attr("class", "items");
        
        // populate items
        for (var i = 0; i < viewData.length; i++) {
            var id = viewData[i];
            var item = model.items[id];
            var differences = [];
            
            var $div = $("<div></div>");
            $div.attr("id", id);
            $div.attr("class", "undecided");
            
            var $span = $("<span></span>");
            $span.attr("class", "item-name");
            $span.text(item.name);
            
            if (id in model.similar) {
                differences = model.similar[id].differences;
                
                if (differences.indexOf(model.ATTR_NAME) != -1) {
                    $span.addClass("difference");
                }
            }
            $div.append($span);
            $div.append("<br />");
            
            for (var attribute in item.attributes) {
                if (model.attributes[attribute].display) {
                    $span = $("<span></span>");
                    $span.text(item.attributes[attribute].toString());
                    
                    if (id in model.similar) {
                        if (differences.indexOf(attribute) != -1) {
                            $span.addClass("difference");
                        }
                    }
                    $div.append($span);
                }
            }
            $items.append($div);
        }
        $container.append($items);
    }
    
    function populateBackdrop($container) {
        // prepare column headers and empty cells
        var $table = $("<table></table>");
        var $thead = $("<thead></thead>");
        
        for (var i = 0; i < NUM_COLUMNS; i++) {
            var $th = $("<th></th>");
            
                // TODO: add explanatory tooltips

            var $accept = $("<a></a>");
            $accept.attr("href", "#");
            $accept.attr("class", "accept-all");
            $accept.text("accept");
            $accept.click({ index: i, accept: true }, processRemaining);

            var $reject = $("<a></a>");
            $reject.attr("href", "#");
            $reject.attr("class", "reject-all");
            $reject.text("reject");
            $reject.click({ index: i, accept: false }, processRemaining);

            var $span = $("<span></span>");
            
            switch(i) {
                case LIST_1_UNIQUE_INDEX:
                    $span.text(model.list1.name + " unique");
                    break;
                case LIST_1_INDEX:
                    $span.text(model.list1.name);
                    break;
                case LIST_IDENTICAL_INDEX:
                    $span.text("Identical");
                    break;
                case LIST_2_INDEX:
                    $span.text(model.list2.name);
                    break;
                case LIST_2_UNIQUE_INDEX:
                    $span.text(model.list2.name + " unique");
                    break;
            }
            $th.append($span);
            $th.append("<br />");
            $th.append($accept);
            $th.append("<span>/</span>");
            $th.append($reject);
            $th.append("<span> remaining</span>");
            $thead.append($th);
            
            // by default, only list 1 and list 2 are visible
            if (i % 2 === 0) {
                toggleHeader($th, false);
            } else {
                toggleHeader($th, true);
            }
        }
        $table.append($thead);
        
        // plug table with empty placeholders (default height is a heuristic)
        var $tbody = $("<tbody></tbody>");

        for (i = 0; i < NUM_COLUMNS * 2; i++) {
            var $tr = $("<tr></tr>");
            
            for (var j = 0; j < NUM_COLUMNS; j++) {
                $tr.append("<td></td>");
            }
            $tbody.append($tr);
        }
        $table.append($thead);
        $table.append($tbody);
        $container.append($table);
        
        // add container for header labels
        var $div = $("<div></div>");
        $div.attr("id", "header-labels");
        $container.append($div);
    }
    
    function prepareHandlers() {
        // item interaction
        $(".items > div").mousedown(clickHandler)
                .mouseover(mouseoverHandler)
                .mouseout(mouseoutHandler)
                .bind("contextmenu", function() {       // enable right click action
                    return false;
                });
        
        // help modal
        $("#help").click(function() {
            
                // TODO: add help
                alert("TODO: integrate with external interface's help");
                
        });
        
        // alert modal
        $("#alert-close").click(function() {
            $(".alert-modal").removeClass("show");
        });
        
        // options panel
        $("#options-panel").hide();
        
        $("#options").click(function() {
            var $options = $("#options");
            
            if ($options.hasClass("selected")) {
                $options.removeClass("selected");
                $("#options-panel").slideUp("fast", function() {
                    redraw(true, true, true);
                });
            } else {
                $options.addClass("selected");
                $("#options-panel").slideDown("fast", function() {
                    redraw(true, true, true);
                });
            }
        });
        
        $("#dataset").change(function() {
            if ($(this).val() !== model.dataset) {
                utils.setStorageItem("dataset", $(this).val());
                location.reload();
            }
        });
        
        $("#groupBy").change(function() {
            model.groupBy = $(this).val();
            
            // force; grouping in compact looks strange
            if (state > STATE_SIMILAR) {
                state = STATE_SIMILAR;
            }
            
            if ($(this).val() === "") {
                $("#compact").removeClass("inactive");
            } else {
                $("#compact").addClass("inactive");
            }
            redraw(true, true);
        });
        
        $("#sortBy").change(function() {
            model.sortBy = $(this).val();
            redraw(true, true);
        });
        
        $("#filterOn").keyup(function() {
            model.filterOn = $(this).val();
            redraw(true, true);
        });
        
        $("#separate").click(function() {
            changeState(STATE_SEPARATE);
        });
        
        $("#identical").click(function() {
            changeState(STATE_IDENTICAL);
        });
        
        $("#unique").click(function() {
            changeState(STATE_UNIQUE);
        });
        
        $("#similar").click(function() {
            changeState(STATE_SIMILAR);
        });
        
        $("#compact").click(function() {
            if (!model.groupBy) {
                changeState(STATE_COMPACT);
            }
        });
        
        $("#grayout").click(function() {
            if (!$(this).hasClass("selected")) {
                $("#remove").removeClass("selected");
                $(this).addClass("selected");
                
                model.afterAction = model.AFTER_ACTION_GRAYOUT;
                redraw(true, true);
            }
        });
        
        $("#remove").click(function() {
            if (!$(this).hasClass("selected")) {
                $("#grayout").removeClass("selected");
                $(this).addClass("selected");
                
                model.afterAction = model.AFTER_ACTION_REMOVE;
                redraw(true, true);
            }
        });
        
        
        // basic controls
        $("#compare").click(function() {
            if (model.groupBy) {
                changeState(STATE_SIMILAR);
            } else {
                changeState(STATE_COMPACT);
            }
        });
        
        $("#confirm").click(function() {
            
                // TODO: connect back to actual interface (check done condition)
                alert("TODO: connect back to actual interface");
        });
        
        // reset
        $("#reset").click(reset);
        $("#logo").click(reset);
        
        // resize with window
        $(window).resize(function() {
            redraw(true, true, true);
        });
    }
    
    
    // event listeners ////////////////////////////////////////////////////////
    function mouseoverHandler(event) {
        var id = $(this).attr("id");
        
        hoverItem(id, true);
        hoverRelated(id, true);
        
        // update item detail
        var item = model.items[id];
        var $detail = $("#detail td:last-child");
        var fullDescription = item.name + " / ";
        
        for (var attributeName in model.attributes) {
            if (attributeName in item.attributes) {
                var values = item.attributes[attributeName];
                var valuesString = "";
                
                if (values.length > 1) {
                    var values = item.attributes[attributeName];
                    
                    for (var i = 0; i < values.length; i++) {
                        
                        if (attributeName === model.ATTR_SUBITEM) {
                            var subitem = values[i];
                            
                            valuesString += subitem.name + " (" +
                                    subitem.attributes[model.ATTR_DOSAGE] +
                                    ")";
                        } else {
                            valuesString += values[i];
                        }
                        valuesString += ", ";
                    }
                    valuesString = valuesString.slice(0, -2);
                } else {
                    valuesString = values.toString();
                }
                fullDescription += valuesString + " / ";
            }
        }
        fullDescription = fullDescription.slice(0, -3);
        $detail.text(fullDescription);
    }
    
    function mouseoutHandler(event) {
        var id = $(this).attr("id");
        
        hoverItem(id, false);
        hoverRelated(id, false);
        
        $("#detail td:last-child").text("Nothing to display.");
    }
    
    function clickHandler(event) {
        
            // TODO: very useful for debugging, keep until final ship
            console.log($(this).attr("id"));
            console.log(model.items[$(this).attr("id")].attributes[model.ATTR_DRUG_CLASS]);
        
        var src, dst;
        
        if (event.which === 1) {            // left click
            if ($(this).hasClass("undecided")) {
                src = "undecided";
                dst = "accepted";
            } else if ($(this).hasClass("accepted")) {
                src = "accepted";
                dst = "rejected";
            } else if ($(this).hasClass("rejected")) {
                src = "rejected";
                dst = "undecided";
            }
        } else if (event.which === 3) {     // right click
            if ($(this).hasClass("undecided")) {
                src = "undecided";
                dst = "rejected";
            } else if ($(this).hasClass("accepted")) {
                src = "accepted";
                dst = "undecided";
            } else if ($(this).hasClass("rejected")) {
                src = "rejected";
                dst = "accepted";
            }
        }
        
        if (src) {
            var id = $(this).attr("id");
            
            processItem(id, src, dst);
        }
    }
    
    function hoverItem(id, mouseover) {
        var $item = $("#" + id);
        
        if (mouseover) {
            $item.addClass("item-hover");
            $item.css("opacity", "1");
            
            if ($item.hasClass("undecided")) {
                $item.addClass("undecided-hover");
            }
        } else {
            $item.removeClass("item-hover");
            
            if ($item.hasClass("undecided")) {
                $item.removeClass("undecided-hover");
            }
        }
    }
    
    function hoverRelated(id, mouseover) {
        var related = (state < STATE_IDENTICAL) ? model.getRelated(id) :
                model.getSimilar(id);
                
        for (var i = 0; i < related.length; i++) {
            hoverItem(related[i], mouseover);
        }
    }
    
    
    // item action ////////////////////////////////////////////////////////////
    function processItem(id, src, dst) {
        var $item = $("#" + id);
            
        if (dst === "undecided") {
            $item.addClass("undecided-hover");
            
            if (state >= STATE_SIMILAR) {
                $item.children(".difference").addClass("highlight");
            }
        } else {
            $item.removeClass("undecided-hover");
            $item.children(".difference").removeClass("highlight");
        }
        $item.removeClass(src);
        $item.addClass(dst);
        
        processRelated(id, src, dst);
    }
    
    function processRelated(id, src, dst) {
        var related = model.getRelated(id);
        
        if (dst === "rejected") {
            var next = id;
            
            // cycle through related items to find next item to accept
            for (var i = 0; i < related.length; i++) {
                var temp = related[i];
                
                if (next < parseFloat(temp)) {
                    next = temp;
                    break;
                }
            }
            
            if (next === id) {
                next = related[0];
            }
            var $item = $("#" + next);
            
            $item.removeClass("undecided accepted rejected undecided-hover");
            $item.addClass("accepted");
        } else {
            for (var i = 0; i < related.length; i++) {
                var relatedID = related[i];
                var $related = $("#" + relatedID);
                $related.removeClass("undecided accepted rejected");
                
                if (dst === "undecided") {
                    $related.addClass("undecided undecided-hover");
                    
                    if (state >= STATE_SIMILAR) {
                        $related.children(".difference").addClass("highlight");
                    }
                } else if (dst === "accepted") {
                    $related.removeClass("undecided-hover");
                    $related.addClass("rejected");
                    $related.children(".difference").removeClass("highlight");
                }
            }
        }
        
        if (model.afterAction === model.AFTER_ACTION_REMOVE) {
            redraw(true, true);
        }
    }
    
    function processRemaining(event) {
        var col = keyframes[state][event.data.index];
        
        for (var i = 0; i < col.length; i++) {
            var id = col[i];
            
            if ($("#" + id).hasClass("undecided")) {
                var dst = (event.data.accept) ? "accepted" : "rejected";
                processItem(id, "undecided", dst);
            }
        }
    }
    
    
    // state and animation ////////////////////////////////////////////////////
    function redraw(sort, filter, immediate) {
        // tweak proportions
        $("#content").height(
            $(window).height() -
            $(".header").height() -
            $("#options-panel").height() -
            $("#detail").height() -
            $(".footer").height()
        );
        
        $(".footer img").css("height", $(".header").height() * 0.75 + "px");
        $("#detail").css("marginTop",
                $("#content").outerHeight(true) + "px");
        
        if (immediate) {
            viewData = model.viewData(sort, filter);
            calculatePositions();
            
            $(".items div").each(function() {
                var id = $(this).attr("id");
                var row = positions[id][state].row;
                var col = positions[id][state].col;
                
                $(this).animate({
                    "left" : ((col) * columnWidth()) + "px",
                    "top" : ((row * rowHeight()) + topOffset()) + "px"
                },
                {
                    duration: buffer / 4,
                    easing: "easeInOutCubic"
                });
            });
        } else {
            viewData = model.viewData(sort, filter);
            calculatePositions();
            changeState(state);
        }
    }
    
    function changeState(toState) {
        var offset = 0;
        
        if (state === toState) {
            transition(state, state);
        } else {
            // force; grouping looks odd in compact
            if (model.groupBy && toState > STATE_SIMILAR) {
                toState = STATE_COMPACT;
            }
            offset = (state < toState) ? 1 : -1;
            var i = state;
            
            while (i !== toState) {
                transition(state, i + offset);
                i += offset;
            }
            state = toState;
        }
        $("#compare").removeClass("inactive");
        $("#separate, #identical, #unique, #similar, #compact")
                .removeClass("selected");
        
        // indicate destination state
        switch(state) {
            case STATE_SEPARATE:
                $("#separate").addClass("selected");
                break;
            case STATE_IDENTICAL:
                $("#identical").addClass("selected");
                break;
            case STATE_UNIQUE:
                $("#unique").addClass("selected");
                break;
            case STATE_SIMILAR:
                $("#similar").addClass("selected");
                break;
            case STATE_COMPACT:
                $("#compact").addClass("selected");
                break;
        }
        
        // handle identical item overlap
        if (state < STATE_IDENTICAL) {
            for (id in model.hidden) {
                if (viewData.indexOf(parseFloat(id)) !== -1) {
                    delete model.hidden[id];
                    model.toggleItem($("#" + id), buffer, true);
                }
            }
        } else {
            for (id in model.identical) {
                if (model.identical[id].indexOf(parseFloat(id)) !== 0 &&
                        viewData.indexOf(parseFloat(id)) !== -1) {
                    model.hidden[id] = true;
                }
            }
        }
    }
    
    function transition(from, to) {
        var delay = 0;
        var offset = Math.abs(to - from);
       
        if (from < to) {
            delay = transitionTime(state, to - 1);
        } else if (from > to) {
            delay = transitionTime(state, to + 1);
        }
        
        setTimeout(function() {
            // adjust table height
            adjustBackdrop(to);
            
            // adjust headers
            adjustHeaders(from, to);
            
            // update row background colors
            var $tbody = $("#content tbody");
            var height = $tbody.children().length;

            for (var i = 0; i < height; i++) {
                $tbody.children(":nth-child(" + (i + 1) + ")")
                        .css("background", "");
            }

            if (model.groupBy) {
                var color = WHITE;
                
                for (var i = 0; i < headers.length; i++) {
                    var header = headers[i];
                    var start = "thresholdRowStart";
                    var end = "thresholdRowEnd";
                    var threshold = STATE_SIMILAR;
    
                    for (var j = 0; j < header.items.length; j++) {
                        var id = header.items[j];
                        
                        if (positions[id].threshold < threshold) {
                            threshold = positions[id].threshold;
                        }
                    }
                    
                    if (state < threshold) {
                        start = "initialRowStart";
                        end = "initialRowEnd";
                    }
                    
                    if (header.hasOwnProperty(start) &&
                            header.hasOwnProperty(end)) {
                        for (var j = header[start]; j < header[end]; j++) {
                            $tbody.children(":nth-child(" + (j + 1) + ")")
                                    .css("background", color);
                        }
                        color = (color === WHITE) ? GRAY : WHITE;
                    }
                }
            } else {
                // stripe rows for legibility
                for (var i = 0; i < height; i++) {
                    if (i % 2 !== 0) {
                        $tbody.children(":nth-child(" + (i + 1) + ")")
                                .css("background", GRAY);
                    }
                }
            }

            // animate column headers
            if (from < to) {
                switch(to) {
                    case STATE_IDENTICAL:
                        var header = $("#content th:nth-child(" +
                                (LIST_IDENTICAL_INDEX + 1) + ")");
                        toggleHeader(header, true);
                        break;
                    case STATE_UNIQUE:
                        var header = $("#content th:nth-child(" +
                                (LIST_1_UNIQUE_INDEX + 1) + ")");
                        toggleHeader(header, true);
                        
                        header = $("#content th:nth-child(" +
                                (LIST_2_UNIQUE_INDEX + 1) + ")");
                        toggleHeader(header, true);
                        
                        header = $("#content th:nth-child(" +
                                (LIST_1_INDEX + 1) + ")");
                        header.children("span:first-child").
                                text(model.list1.name + " similar");

                        header = $("#content th:nth-child(" +
                                (LIST_2_INDEX + 1) + ")");
                        header.children("span:first-child").
                                text(model.list2.name + " similar");
                        break;
                    case STATE_SIMILAR:
                        var header = $("#content th:nth-child(" +
                                (LIST_1_INDEX + 1) + ")");
                        
                        header = $("#content th:nth-child(" +
                                (LIST_2_INDEX + 1) + ")");
                        
                        $(".difference").each(function() {
                            if ($(this).parent().hasClass("undecided")) {
                                $(this).addClass("highlight");
                            }
                        });
                        break;
                }
            } else {
                switch(to) {
                    case STATE_SEPARATE:
                        var header = $("#content th:nth-child(" +
                                (LIST_IDENTICAL_INDEX + 1) + ")");
                        
                        for (var i = 0; i < NUM_COLUMNS; i++) {
                            var header = $("#content th:nth-child(" + (i + 1)
                                    + ")");
                                    
                            if (i % 2 == 0) {
                                toggleHeader(header, false);
                            } else {
                                toggleHeader(header, true);
                            }
                        }
                        break;
                    case STATE_IDENTICAL:
                        var header = $("#content th:nth-child(" +
                                (LIST_1_UNIQUE_INDEX + 1) + ")");
                        toggleHeader(header, false);
                        
                        header = $("#content th:nth-child(" +
                                (LIST_2_UNIQUE_INDEX + 1) + ")")
                        toggleHeader(header, false);
                        
                        var header = $("#content th:nth-child(" +
                                (LIST_1_INDEX + 1) + ")");
                        header.children("span:first-child")
                                .text(model.list1.name);
                        
                        header = $("#content th:nth-child(" +
                                (LIST_2_INDEX + 1) + ")");
                        header.children("span:first-child")
                                .text(model.list2.name);
                        
                        header = $("#content th:nth-child(" +
                                (LIST_IDENTICAL_INDEX + 1) + ")");
                        toggleHeader(header, true);
                        break;
                    case STATE_UNIQUE:
                        var header = $("#content th:nth-child(" +
                                (LIST_1_INDEX + 1) + ")");
                        
                        header = $("#content th:nth-child(" +
                                (LIST_2_INDEX + 1) + ")");
                        
                        $(".difference").each(function() {
                            if ($(this).parent().hasClass("undecided")) {
                                $(this).removeClass("highlight");
                            }
                        });
                        break;
                }
            }
            
            // animate items
            if (to > from) {
                if (to === STATE_IDENTICAL) {
                    animateIdentical(to);
                } else if (to === STATE_UNIQUE) {
                    animateUnique(to);
                } else {
                    animateDefault(to);
                }
            } else {
                animateDefault(to);
            }
        }, delay);
    }
    
    function animateIdentical(toState) {
        var i = 0;
        var checked = {}, animated = {};
        
        for (var id in model.identical) {
            if (checked[id] === undefined) {
                var set = model.identical[id];
                checked[id] = true;
                
                for (var j = 0; j < set.length; j++) {
                    checked[set[j]] = true;
                }
                animateSet(set, toState,
                        (i > 0) ? (i * animationSpeed) + (i * buffer) : 0,
                        animated);
                i++;
            }
        }
    }
    
    function animateUnique(toState) {
        // animate left first, then right
        animateSet(model.unique1, toState, 0)
        animateSet(model.unique2, toState, animationSpeed + buffer);
    }
    
    function animateDefault(toState) {
        $(".items div").each(function() {
            animateItem($(this).attr("id"), toState);
        });
    }
    
    function animateSet(set, toState, delay, animated) {
        setTimeout(function() {
            for (var i = 0; i < set.length; i++) {
                var id = set[i];
                
                if (animated) {
                    if (animated[id] === undefined) {
                        animated[id] = true;
                        animateItem(id, toState);
                    }
                } else {
                    animateItem(id, toState);
                }
            }
        }, delay);
    }
    
    function animateItem(id, toState) {
        var $item = $("#" + id);
        var row = positions[id][toState].row;
        var col = positions[id][toState].col;
        
        $item.animate({
            "left" : ((col) * columnWidth()) + "px",
            "top" : ((row * rowHeight()) + topOffset()) + "px"
        },
        {
            duration: animationSpeed,
            easing: "easeInOutCubic"
        });
        
        if (id in model.hidden && state >= STATE_IDENTICAL) {
            model.toggleItem($item, buffer, false);
        }
    }
    
    function toggleHeader($header, show) {
        $header.children().toggle(show);
    }
    
    function topOffset() {
        return $("#content th").outerHeight(true);
    }
    
    function columnWidth() {
        return $("#content td").outerWidth(true);
    }
    
    function rowHeight() {
        var $td = $("#content td");
        var td = $td.height();
        var padding = parseFloat($td.css("paddingTop")) + 
                parseFloat($td.css("paddingBottom"));
        
        return td + padding;
    }
    
    function transitionTime(from, to) {
        var delay = 0;

        if (from < to) {
            for (i = from + 1; i <= to; i++) {
                if (i === STATE_IDENTICAL) {
                    delay += transitionIdenticalTime();
                } else if (i === STATE_UNIQUE) {
                    delay += 2 * (animationSpeed + buffer);
                } else {
                    delay += animationSpeed + buffer;
                }
            }
        } else if (from > to) {
            for (i = from; i >= to + 1; i--) {
                delay += animationSpeed + buffer;
            }
        }
        return delay;
    }
    
    function transitionIdenticalTime() {
        var numSets = 0;
        var checked = {};
        
        for (var id in model.identical) {
            if (checked[id] === undefined) {
                var set = model.getIdentical(id);
                checked[id] = true;
                
                for (var j = 0; j < set.length; j++) {
                    checked[set[j]] = true;
                }
                numSets++;
            }
        }
        return (numSets * animationSpeed) + (numSets * buffer);
    }
    
    
    // position calculation ///////////////////////////////////////////////////
    function calculateColumns() {
        for (var i = 0; i < viewData.length; i++) {
            var id = viewData[i];
            var side = (model.items[id].listID === model.list1.id) ? -1 : 1;
            var threshold = STATE_UNIQUE;
            var initialCol = Math.floor(NUM_COLUMNS / 2) + side;
            var finalCol = initialCol + side;
            
            if (id in model.similar) {
                threshold = STATE_SIMILAR;
                finalCol = initialCol;
            }
            
            if (id in model.identical) {
                threshold = STATE_IDENTICAL;
                finalCol = Math.floor(NUM_COLUMNS / 2);
                
                /*
                 * If all members of an identical set belong to the same list,
                 * simply merge them; the middle column is reserved for items
                 * that are identical among both lists.
                 */
                var identical = model.getIdentical(id);
                
                for (var j = 0; j < identical.length; j++) {
                    if (model.items[identical[j]].listID !==
                            model.items[id].listID) {
                        break;
                    }
                }
                
                if (j === identical.length) {
                    finalCol = initialCol;
                    
                        // TODO: hack, should really support STATE_IDENTICAL
                    
                    threshold = STATE_SIMILAR;
                }
            }
            positions[id] = {
                        threshold: threshold,
                        initialCol: initialCol,
                        finalCol: finalCol
                    };
        }
    }
    
    function calculatePositions() {
        // clear rows
        for (var i = 0; i < viewData.length; i++) {
            var position = positions[viewData[i]];
            
            if (position.hasOwnProperty("initialRow")) {
                delete position.initialRow;
            }
            
            if (position.hasOwnProperty("thresholdRow")) {
                delete position.thresholdRow;
            }
            
            if (position.hasOwnProperty("finalRow")) {
                delete position.finalRow;
            }
        }
        
        // (re-)calculate
        calculateRows();
        calculateKeyframes();
    }
    
    function calculateRows() {
        var initialRows = [0, 0];
        var thresholdRows = [], finalRows = [];
        var bumped = [];
        
        for (var i = 0; i < NUM_COLUMNS; i++) {
            thresholdRows.push(0);
            finalRows.push(0);
        }
        
        // reset headers
        headers = [];
        currentHeader = "";
        previousHeader = "";
        
        for (i = 0; i < viewData.length; i++) {
            var id = viewData[i];
            var position = positions[id];
            
                // TODO: properly account for compact group positions
            
            // impose group restriction on spatial position
            if (model.groupBy) {
                previousHeader = currentHeader;
                currentHeader = model.items[id].attributes[model.groupBy][0];
                
                if (currentHeader.toLowerCase() !== previousHeader.toLowerCase()) {
                    var row = Math.max(initialRows[0], initialRows[1]);
                    initialRows[0] = row;
                    initialRows[1] = row;
                    
                    var thresholdRow = thresholdRows[0];
                    var finalRow = finalRows[0];
                    
                    for (var j = 1; j < NUM_COLUMNS; j++) {
                        thresholdRow = Math.max(thresholdRow, thresholdRows[j]);
                        finalRow = Math.max(finalRow, finalRows[j]);
                    }
                    
                    for (j = 0; j < NUM_COLUMNS; j++) {
                        thresholdRows[j] = thresholdRow;
                        finalRows[j] = finalRow;
                    }
                    
                    if (previousHeader.length > 0) {
                        var previous = headers[headers.length - 1];
                        
                        previous.initialRowEnd = row;
                        previous.thresholdRowEnd = thresholdRow;
                    }
                    headers.push({
                        label: currentHeader,
                        initialRowStart: row,
                        thresholdRowStart: thresholdRow,
                        items: [id]
                    });
                } else {
                    headers[headers.length - 1].items.push(id);
                }
            }
            position.initialRow =
                    (model.items[id].listID === model.list1.id) ?
                    initialRows[0]++ : initialRows[1]++;
            calculateRow(id, "thresholdRow", thresholdRows);
            calculateRow(id, "finalRow", finalRows, bumped);
        }
        
        // re-align bumped-down items
        var height = Math.max(finalRows[LIST_1_UNIQUE_INDEX],
                finalRows[LIST_1_INDEX],
                finalRows[LIST_IDENTICAL_INDEX],
                finalRows[LIST_2_INDEX],
                finalRows[LIST_2_UNIQUE_INDEX]);
                
        for (i = 0; i < NUM_COLUMNS; i++) {
            finalRows[i] = height;
        }
        
        for (i = 0; i < bumped.length; i++) {
            calculateRow(bumped[i], "finalRow", finalRows);
        }
        
        if (model.groupBy && headers.length > 0) {
            var header = headers[headers.length - 1];
            
            header.initialRowEnd = Math.max(initialRows[0], initialRows[1]);
            header.thresholdRowEnd = Math.max(thresholdRows[LIST_1_UNIQUE_INDEX],
                thresholdRows[LIST_1_INDEX],
                thresholdRows[LIST_IDENTICAL_INDEX],
                thresholdRows[LIST_2_INDEX],
                thresholdRows[LIST_2_UNIQUE_INDEX]);
            header.finalRowEnd = height;
        }
    }
    
    function calculateRow(id, type, rows, bumped) {
        var position = positions[id];
        var identical = model.getIdentical(id);
        var similar = model.getSimilar(id);
        var bump = false;
        
        // row already calculated
        if (position.hasOwnProperty(type)) {
            return;
        }
        
        // bump similar items down
        if (bumped) {
            if (id in model.similar) {
                bump = true;
            } else if (id in model.identical) {
                for (var i = 0; i < identical.length; i++) {
                    if (identical[i] in similar) {
                        bump = true;
                        break;
                    }
                }
            }
        }
        
        if (bump) {
            bumped.push(id);
            
            return;
        }
        
        // position self
        position[type] = rows[position.finalCol];
        
        // position identical items
        for (i = 0; i < identical.length; i++) {
            var relatedPosition = positions[identical[i]];
            
            if (!relatedPosition.hasOwnProperty(type)) {
                relatedPosition[type] = position[type];
            }
        }
        
        // position similar items
        rows[position.finalCol]++;
        
        for (i = 0; i < similar.length; i++) {
            relatedPosition = positions[similar[i]];
            
            if (!relatedPosition.hasOwnProperty(type)) {
                if (model.groupBy) {
                    if (model.items[similar[i]].attributes[model.groupBy][0] ===
                            currentHeader) {
                        relatedPosition[type] =
                                rows[relatedPosition.finalCol]++;
                    }
                } else {
                    relatedPosition[type] = rows[relatedPosition.finalCol]++;
                }
            }
        }
        
        // re-align
        if (type === "thresholdRow") {
            var height = Math.max(rows[LIST_1_UNIQUE_INDEX],
                    rows[LIST_1_INDEX], rows[LIST_IDENTICAL_INDEX],
                    rows[LIST_2_INDEX], rows[LIST_2_UNIQUE_INDEX]);
                    
            for (i = 0; i < NUM_COLUMNS; i++) {
                rows[i] = height;
            }
        } else {
            var height = Math.max(rows[LIST_1_INDEX],
                    rows[LIST_IDENTICAL_INDEX], rows[LIST_2_INDEX]);
            
            for (i = LIST_1_INDEX; i <= LIST_2_INDEX; i++) {
                rows[i] = height;
            }
        }
    }
    
    function calculateKeyframes() {
        keyframes = {};
        
        for (var i = 0; i < NUM_STATES; i++) {
            keyframes[i] = [];
            NUM_ROWS[i] = 0;
            
            for (var j = 0; j < NUM_COLUMNS; j++) {
                keyframes[i][j] = [];
            }
        }
        
        for (i = 0; i < viewData.length; i++) {
            var id = viewData[i];
            var position = positions[id];
            
            // initial position
            for (j = 0; j < position.threshold; j++) {
                position[j] = {
                            row: position.initialRow,
                            col: position.initialCol
                        };
                keyframes[j][position.initialCol].push(id);
                NUM_ROWS[j] = Math.max(NUM_ROWS[j], position.initialRow + 1);
            }
            
            // threshold position
            for ( ; j < STATE_COMPACT; j++) {
                position[j] = {
                            row: position.thresholdRow,
                            col: position.finalCol
                        };
                keyframes[j][position.finalCol].push(id);
                NUM_ROWS[j] = Math.max(NUM_ROWS[j], position.thresholdRow + 1);
            }
            
            // final (compact) position
            position[j] = {
                        row: position.finalRow,
                        col: position.finalCol
                    };
            keyframes[j][position.finalCol].push(id);
            NUM_ROWS[j] = Math.max(NUM_ROWS[j], position.finalRow + 1);
        }
    }
    
    function adjustBackdrop(toState) {
        var $tbody = $("#content > table > tbody");
        var currentHeight = $tbody.children("tr").length;
        var newHeight = NUM_ROWS[toState];
        
        if (currentHeight < newHeight) {
            var difference = newHeight - currentHeight;
            
            for (var i = 0; i < difference; i++) {
                var $tr = $("<tr></tr>");
                
                for (var j = 0; j < NUM_COLUMNS; j++) {
                    $tr.append("<td></td>");
                }
                $tbody.append($tr);
            }
        } else if (currentHeight > newHeight) {
            for (var i = currentHeight; i > newHeight; i--) {
                $tbody.children(":nth-child(" + i +")").remove();
            }
        }
    }
    
    function adjustHeaders(from, to) {
        $("#header-labels span").remove();
        adjustCellPadding();
        
        if (model.groupBy) {
            for (var i = 0; i < headers.length; i++) {
                var header = headers[i];
                var row = header.thresholdRowStart;
                var $span = $("<span></span>");
                var threshold = STATE_SIMILAR;

                for (var j = 0; j < header.items.length; j++) {
                    var id = header.items[j];
                    
                    if (positions[id].threshold < threshold) {
                        threshold = positions[id].threshold;
                    }
                }
                
                if (state < threshold) {
                    row = header.initialRowStart;
                }
                $span.attr("class", "header-label");
                $span.text(header.label[0].toUpperCase() +
                        header.label.slice(1));
                $span.css("left", "0");
                $span.css("top", topOffset() + (row * rowHeight()) + "px");
                $("#header-labels").append($span);
            }
        }
    }
    
    function adjustCellPadding() {
        if (model.groupBy) {
            $(".items div").css("paddingTop", "0.9em");
            $(".items div").css("paddingBottom", "0.1em");
            $(".items div").css("paddingLeft", "0.5em");
            $(".items div").css("paddingRight", "0.5em");
        } else {
            $(".items div").css("paddingTop", "0.6em");
            $(".items div").css("paddingBottom", "0.4em");
            $(".items div").css("paddingLeft", "0.5em");
            $(".items div").css("paddingRight", "0.5em");
        }
    }
    
    // expose interface ///////////////////////////////////////////////////////
    return visible;
}(window.controller = window.controller || {}, $, undefined);
