/*
 * FILENAME:    model.js
 * VERSION:     0.256 (10/18/2011)
 * AUTHOR:      Tiffany Chao (tchao@cs.umd.edu)
 */

var model = function(model, jQuery) {
////// visible ////////////////////////////////////////////////////////////////
    var visible = {};
    
    // pseudo constants
    visible.ATTR_NAME = "__ATTR_NAME__";
    visible.ATTR_ROUTE = "__ATTR_ROUTE__";
    visible.ATTR_FREQUENCY = "__ATTR_FREQUENCY__";
    visible.ATTR_DOSAGE = "__ATTR_DOSAGE__";
    visible.ATTR_DRUG_CLASS = "__ATTR_DRUG_CLASS__";
    visible.ATTR_SUBITEM = "__ATTR_SUBITEM__";
    
    visible.ATTR_TYPE_NUMERIC = "__ATTR_TYPE_NUMERIC__";
    visible.ATTR_TYPE_GENERAL = "__ATTR_TYPE_GENERAL__";
    visible.ATTR_TYPE_CATEGORICAL = "__ATTR_TYPE_CATEGORICAL__";
    
    visible.DATASET_DEMO = "__DATASET_DEMO__";
    visible.DATASET_SIMPLE = "__DATASET_SIMPLE__";
    visible.DATASET_JORGE = "__DATASET_JORGE__";
    visible.DATASET_COMPLEX = "__DATASET_COMPLEX__";
    visible.DATASET_DEFAULT = visible.DATASET_SIMPLE;
    
    // data
    visible.dataset = "";
    visible.items = {};
    visible.hidden = {};
    visible.list1 = {
                id: "list0",
                name: "Intake",
                source: []
            };
    visible.list2 = {
                id: "list1",
                name: "Hospital",
                source: []
            };
    visible.viewData = function(sort, filter) {
        var viewData = visible.list1.source.concat(visible.list2.source);
        
        if (filter) {
            viewData = viewData.filter(unifiedFilter);
        }
        
        if (sort || filter) {
            viewData = viewData.sort(groupThenSort);
        }
        return viewData;
    }
    
    visible.attributes = {};
    visible.groupBy = "";
    visible.sortBy = "";
    visible.filterOn = "";
    
    visible.AFTER_ACTION_GRAYOUT = "__AFTER_ACTION_GRAYOUT__";
    visible.AFTER_ACTION_REMOVE = "__AFTER_ACTION_REMOVE__";
    visible.afterAction = visible.AFTER_ACTION_GRAYOUT;
    
    visible.unique1 = [];
    visible.unique2 = [];
    visible.identical = {};
    visible.similar = {};
    
    
    // methods
    visible.init = function(dataset) {
        loadData(dataset);
    };
    

        // TODO: group-based exclusion
    
    visible.getIdentical = function(id) {
        var identical = [];
        
        if (id in visible.identical && [id].filter(nameFilter).length > 0) {
            identical = visible.identical[id].slice();
            identical.splice(identical.indexOf(parseFloat(id)), 1);
        }
        return identical.filter(unifiedFilter);
    };
    
    visible.getSimilar = function(id) {
        var similar = [];
        
        if (id in visible.similar && [id].filter(nameFilter).length > 0) {
            similar = visible.similar[id].items.slice();
            similar.splice(similar.indexOf(parseFloat(id)), 1);
        }
        return similar.filter(unifiedFilter);
    };
    
    visible.getRelated = function(id) {
        var identical = visible.getIdentical(id);
        var similar = visible.getSimilar(id);
        var hash = {};
        var length = Math.min(identical.length, similar.length); 
        
        // remove duplicates
        for (var i = 0; i < length; i++) {
            hash[identical[i]] = true;
            hash[similar[i]] = true;
        }
        
        if (length < identical.length) {
            for ( ; i < identical.length; i++) {
                hash[identical[i]] = true;
            }
        } else if (length < similar.length) {
            for ( ; i < similar.length; i++) {
                hash[similar[i]] = true;
            }
        }
        
        // convert results into array format
        var related = [];
        
        for (var id in hash) {
            related.push(id);
        }
        return related;
    };
    
    
    visible.toggleItem = function($item, delay, show) {
        if (show) {
            if (visible.hidden[$item.attr("id")] === undefined) {
                $item.show();
                
                setTimeout(function() {
                    $item.css("opacity", "1");
                }, delay);
            }
        } else {
            setTimeout(function() {
                $item.css("opacity", "0");
                
                setTimeout(function() {
                    $item.hide();
                    $item.removeClass("undecided-hover item-hover");
                }, delay);
            }, delay);
        }
    };
    
    
////// hidden /////////////////////////////////////////////////////////////////
    var fadeDelay = 200;
    
    // initialization
    function loadData(dataset) {
        visible.dataset = dataset;
        populateLists(dataset);
        detectAttributes();
        detectRelationships(dataset);
    }
    
    function populateLists(dataset) {
        var i = 0, attributes = {}, item = {};
        
        if (dataset === visible.DATASET_DEMO) {
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID PRN"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Allegra",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID PRN"];
            attributes[visible.ATTR_DOSAGE] = ["60 / 120 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine",
                    "decongestant"];
            attributes[visible.ATTR_SUBITEM] = [
                        {
                            name: "Fexofenadine",
                            attributes: { "__ATTR_DOSAGE__" : ["60 mg"] }
                        },
                        {
                            name: "Pseudoephedrine",
                            attributes: { "__ATTR_DOSAGE__" : ["120 mg"] }
                        }
                    ];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Allegra-D",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q6h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Tylenol",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            // populate list 2
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["20 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["corticosteroid"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Prednisone",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["PRN"];
            attributes[visible.ATTR_DOSAGE] = ["25 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Dipenhydramine",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
        } else if (dataset === visible.DATASET_SIMPLE) {
            // populate list 1
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q6h"];
            attributes[visible.ATTR_DOSAGE] = ["32 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SC"];
            attributes[visible.ATTR_FREQUENCY] = ["qFriday"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ESA"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Darbepoetin",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["0.25 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["supplement"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Calcitrol",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["5 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ACE inhibitor"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Ramipril",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["7.5 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "NSAID"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Meloxicam",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["1 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["supplement"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Folvite",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            // populate list 2
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SC"];
            attributes[visible.ATTR_FREQUENCY] = ["qFriday"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ESA"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Darbepoetin",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["1 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["supplement"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Folic acid",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["40 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["proton pump inhibitor"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Omeprazole",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["500 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antibiotic"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Ciproflaxocin",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["5 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ACE inhibitor"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Ramipril",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["0.25 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["supplement"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Calcitrol",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["TID"];
            attributes[visible.ATTR_DOSAGE] = ["300 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["supplement"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Ferrous Gloconate",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
        } else if (dataset === visible.DATASET_JORGE) {
            // populate list 1
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["40 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ulcer risk reduction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Nexium",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["25 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Toprol-XL",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["40 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["hypocholesterolemia"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Lipitor",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            

            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID"];
            attributes[visible.ATTR_DOSAGE]= ["20 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Enalapril",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["dieb. alt."];
            attributes[visible.ATTR_DOSAGE]= ["200 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Amiodarone",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SL"];
            attributes[visible.ATTR_FREQUENCY] = ["PRN"];
            attributes[visible.ATTR_DOSAGE]= ["0.4 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Nitrostat",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["75 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Plavix",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["TID"];
            attributes[visible.ATTR_DOSAGE]= ["35 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["anemia"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Ferrous sulfate",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;

            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Aspirin",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["PRN"];
            attributes[visible.ATTR_DOSAGE]= ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["pain management"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Tylenol",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["TID"];
            attributes[visible.ATTR_DOSAGE]= ["30 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
                       
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Isosorbide dinitrate",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID"];
            attributes[visible.ATTR_DOSAGE]= ["2.5 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
                       
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Amlodipine",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            // populate list 2
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["1 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["nutritional deficiency"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Folic acid",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID"];
            attributes[visible.ATTR_DOSAGE]= ["20 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Enalapril",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["25 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Toprol-XL",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE]= ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Aspirin",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID or TID"];
            attributes[visible.ATTR_DOSAGE]= ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["anemia"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Ferrous sulfate",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["HS"];
            attributes[visible.ATTR_DOSAGE]= ["80 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["hypocholesterolemia"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Lipitor",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["dieb. alt."];
            attributes[visible.ATTR_DOSAGE]= ["200 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Amiodarone",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID"];
            attributes[visible.ATTR_DOSAGE]= ["2.5 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
                       
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Amlodipine",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SL"];
            attributes[visible.ATTR_FREQUENCY] = ["PRN"];
            attributes[visible.ATTR_DOSAGE]= ["0.4 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Nitroglycerin",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["TID"];
            attributes[visible.ATTR_DOSAGE]= ["30 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["post-myocardial infarction"];
                       
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Isosorbide dinitrate",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
        } else if (dataset === visible.DATASET_COMPLEX) {
            // populate list 1
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q6h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Tylenol",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["10 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Zyrtec",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["BID"];
            attributes[visible.ATTR_DOSAGE] = ["60 / 120 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine", "decongestant"];
            attributes[visible.ATTR_SUBITEM] = [
                        {
                            name: "Fexofenadine",
                            attributes: { "__ATTR_DOSAGE__" : ["60 mg"] }
                        },
                        {
                            name: "Pseudoephedrine",
                            attributes: { "__ATTR_DOSAGE__" : ["120 mg"] }
                        }
                    ];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Allegra-D",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SC"];
            attributes[visible.ATTR_FREQUENCY] = ["qFriday"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ESA"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Darbepoetin",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q6h"];
            attributes[visible.ATTR_DOSAGE] = ["30 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["decongestant"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Sudafed",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
    
    
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["81 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["salicylate"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Aspirin",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["10 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Claritin",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["200 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["NSAID"];
            
            item = {
                        listID: visible.list1.id,
                        id: i,
                        name: "Advil",
                        attributes: attributes
                    };
            visible.list1.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            // populate list 2
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["antihistamine"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Fexofenadine",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["3325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["200 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["NSAID"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Ibuprofen",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q4h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["daily"];
            attributes[visible.ATTR_DOSAGE] = ["30 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["corticosteroid"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Prednisone",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["PO"];
            attributes[visible.ATTR_FREQUENCY] = ["q6h"];
            attributes[visible.ATTR_DOSAGE] = ["325 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["analgesic", "antipyretic"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Acetaminophen",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
            
            
            attributes = {};
            attributes[visible.ATTR_ROUTE] = ["SC"];
            attributes[visible.ATTR_FREQUENCY] = ["qFriday"];
            attributes[visible.ATTR_DOSAGE] = ["60 mg"];
            attributes[visible.ATTR_DRUG_CLASS] = ["ESA"];
            
            item = {
                        listID: visible.list2.id,
                        id: i,
                        name: "Darbepoetin",
                        attributes: attributes
                    };
            visible.list2.source.push(item.id);
            visible.items[item.id] = item;
            i++;
        }
    }
    
    function detectAttributes() {
        visible.attributes[visible.ATTR_NAME] = {
                    type: visible.ATTR_TYPE_GENERAL,
                    display: true
                };
        visible.attributes[visible.ATTR_ROUTE] = {
                    type: visible.ATTR_TYPE_CATEGORICAL,
                    display: true
                };
        visible.attributes[visible.ATTR_FREQUENCY] = {
                    type: visible.ATTR_TYPE_GENERAL,
                    display: true
                };
        visible.attributes[visible.ATTR_DOSAGE] = {
                    type: visible.ATTR_TYPE_NUMERIC,
                    display: true
                };
        visible.attributes[visible.ATTR_DRUG_CLASS] = {
                    type: visible.ATTR_TYPE_CATEGORICAL,
                    display: false
                };
        visible.attributes[visible.ATTR_SUBITEM] = {
                    type: visible.ATTR_TYPE_GENERAL,
                    display: false
                };
    }
    
    function detectRelationships(dataset) {
        var i, identical, similar;
        
        if (dataset === visible.DATASET_DEMO) {
            visible.unique1 = [2];
            visible.unique2 = [3, 4];
            
            identical = [];
            
            similar = [
                    {
                        items: [0, 1],
                        differences: [visible.ATTR_NAME, visible.ATTR_DOSAGE]
                    }
                    ];
        } else if (dataset === visible.DATASET_SIMPLE) {
            visible.unique1 = [4];
            visible.unique2 = [9, 10, 13];
            
            identical = [
                    [1, 7],
                    [2, 12],
                    [3, 11]
                    ];
            
            similar = [
                    {
                        items: [0, 6],
                        differences: [visible.ATTR_FREQUENCY,
                                visible.ATTR_DOSAGAE]
                    },
                    {
                        items: [5, 8],
                        differences: [visible.ATTR_NAME]
                    }
                    ];
        } else if (dataset === visible.DATASET_JORGE) {
            visible.unique1 = [0, 6, 9];
            visible.unique2 = [12];
            
            identical = [
                    [1, 14],
                    [3, 13],
                    [4, 18],
                    [8, 15],
                    [10, 21],
                    [11, 19]
                    ];
                    
            similar = [
                    {
                        items: [2, 17],
                        differences: [visible.ATTR_FREQUENCY,
                                visible.ATTR_DOSAGE]
                    },
                    {
                        items: [5, 20],
                        differences: [visible.ATTR_NAME]
                    },
                    {
                        items: [7, 16],
                        differences: [visible.ATTR_FREQUENCY,
                                visible.ATTR_DOSAGE]
                    }
                    ];
            
        } else if (dataset === visible.DATASET_COMPLEX) {
            visible.unique1 = [2, 7, 8];
            visible.unique2 = [14];
            
            identical = [
                    [0, 4, 13],
                    [5, 16]
                    ];
            
            similar = [
                    {
                        items: [0, 1, 4, 11, 13, 15],
                        differences: [visible.ATTR_NAME,
                                visible.ATTR_FREQUENCY, visible.ATTR_DOSAGE]
                    },
                    {
                        items: [9, 12],
                        differences: [visible.ATTR_NAME]
                    },
                    {
                        items: [3, 6, 10],
                        differences: [visible.ATTR_NAME,
                                visible.ATTR_FREQUENCY, visible.ATTR_DOSAGE]
                    }];
        }
        
        if (identical || similar) {
            for (i = 0; i < identical.length; i++) {
                var set = identical[i];
                
                for (var j = 0; j < set.length; j++) {
                    visible.identical[set[j]] = set;
                }
            }
            
            for (i = 0; i < similar.length; i++) {
                set = similar[i];
                
                for (j = 0; j < set.items.length; j++) {
                    visible.similar[set.items[j]] = set;
                }
            }
        }
    }
    
    
    // sort
    function groupThenSort(a, b) {
        var groupOrder = visible.groupBy ?
                attributeSort(a, b, visible.groupBy) : 0;
        
        if (groupOrder === 0) {
            var sortOrder = visible.sortBy ?
                    attributeSort(a, b, visible.sortBy) :
                    visible.items[a].id - visible.items[b].id;
                    
            if (visible.sortBy && sortOrder === 0) {
                return visible.items[a].id - visible.items[b].id;
            }
            return sortOrder;
        }
        return groupOrder;
    }
    
    function attributeSort(a, b, attribute) {
        var attributeA;
        var attributeB;
        
        if (attribute === visible.ATTR_NAME) {
            attributeA = visible.items[a].name;
            attributeB = visible.items[b].name;
        } else {
            attributeA = visible.items[a].attributes[attribute][0];
            attributeB = visible.items[b].attributes[attribute][0];
        }
        
        if (visible.attributes[attribute].type === visible.ATTR_TYPE_NUMERIC) {
            attributeA = parseFloat(attributeA);
            attributeB = parseFloat(attributeB);
        }
        
        if (attributeA < attributeB)
            return -1;
        else if (attributeA > attributeB)
            return 1;
        else
            return 0;
    }
    
    
    // filter
    function actionFilter(element, index, array) {
        var $item = $("#" + element);
        
        if (visible.afterAction === visible.AFTER_ACTION_REMOVE &&
                !$item.hasClass("undecided")) {
            return false;
        }
        return true;
    }
    
    function nameFilter(element, index, array) {
        var $item = $("#" + element);
        
        visible.filterOn = visible.filterOn.toLowerCase();
        
        if (visible.filterOn.length > 0 &&
                visible.items[element].name.toLowerCase()
                .indexOf(visible.filterOn) === -1) {
            return false;
        }
        return true;
    }
    
    function unifiedFilter(element, index, array) {
        var keep = actionFilter(element, index, array);
        
        if (keep) {
            keep = nameFilter(element, index, array);
        }
        visible.toggleItem($("#" + element), fadeDelay, keep);
        
        return keep;
    }
    
    
    // expose interface ///////////////////////////////////////////////////////
    return visible;
}(window.model = window.model || {}, $, undefined);
